@@ -131,7 +131,7 @@ soup_headers_clean_for_10 (SoupMessageHeaders *hdrs)
 	const char *connection;
 	GSList *tokens, *t;
 
-	connection = soup_message_headers_get (hdrs, "Connection");
+	connection = soup_message_headers_get_list (hdrs, "Connection");
 	if (!connection)
 		return;
 
@@ -787,10 +787,11 @@ append_param_quoted (GString *string, const char *value)
  * soup_header_g_string_append_param:
  * @string: a #GString being used to construct an HTTP header value
  * @name: a parameter name
- * @value: a parameter value
+ * @value: a parameter value, or %NULL
  *
- * Appends something like <literal>@name="@value"</literal> to @string,
- * taking care to appropriately escape any quotes or backslashes in @value.
+ * Appends something like <literal>@name="@value"</literal> to
+ * @string, taking care to appropriately escape any quotes or
+ * backslashes in @value.
  *
  * Alternatively, if @value is a non-ASCII UTF-8 string, it will be
  * appended using RFC2231 syntax. Although in theory this is supposed
@@ -798,6 +799,8 @@ append_param_quoted (GString *string, const char *value)
  * reality, it can only be used portably with the Content-Disposition
  * "filename" parameter.
  *
+ * If @value is %NULL, this will just append @name to @string.
+ *
  * Since: 2.26
  **/
 void
@@ -807,6 +810,8 @@ soup_header_g_string_append_param (GString *string, const char *name,
 	const char *v;
 
 	g_string_append (string, name);
+	if (!value)
+		return;
 
 	for (v = value; *v; v++) {
 		if (*v & 0x80) {
